-- ***************************************************************************
-- Script de creaci dels elements de la interfcie pblica de la base de dades UOC_SGDB
-- Autor: Miguel ngel Bueno Snchez
-- TFC BBDD Relacionals
-- 2014-2015-1
-- ***************************************************************************
set serveroutput on;

-- ---------------------------------------------------------------------------
-- Procediment per crear un nou ciutad a la base de dades.
-- Parmetres:
-- pCountryCode     IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pFirstName       IN VARCHAR2   Nom del ciutad.
-- pLastName        IN VARCHAR2   Cognoms del ciutad.
-- pSex             IN CHAR       Sexe del ciutad. Noms els valors H (home) i D (dona).
-- pEmail           IN VARCHAR2   Adrea de correu electrnic del ciutad. Pot ser null.
-- pBirthday        IN DATE       Data de naixement.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE CreateCitizen
(
  pCountryCode IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdNumber IN system.CIUTADANS."IDCIUTADA"%TYPE,
  pFirstName IN system.CIUTADANS."NOM"%TYPE,
  pLastName IN system.CIUTADANS."COGNOMS"%TYPE,
  pSex IN system.CIUTADANS."SEXE"%TYPE,
  pEmail IN system.CIUTADANS."EMAIL"%TYPE,
  pBirthday IN system.CIUTADANS."DATANAIXEMENT"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_IdCiutada EXCEPTION;
  error_NomCiutada EXCEPTION;
  error_CognomsCiutada EXCEPTION;
  error_SexeCiutada EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'CreateCitizen';
  vParametres := pCountryCode || ', ' || pIdNumber || ', ' || pFirstName || ', ' || pLastName || ', ' 
                 || pSex || ', ' || pEmail || ', ' || pBirthday;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    if (pIdNumber IS NULL or pIdNumber = '') then
      raise error_IdCiutada; 
    elsif (pFirstName IS NULL or pFirstName = '') then
      raise error_NomCiutada; 
    elsif (pLastName IS NULL or pLastName = '') then
      raise error_CognomsCiutada; 
    elsif (pSex IS NULL or (pSex <> 'H' and pSex <> 'D')) then
      raise error_SexeCiutada; 
    end if;
      
    -- Crear el ciutad.
    procCrearCiutada(pCountryCode, pIdNumber, pFirstName, pLastName, pSex, pEmail, pBirthday, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_IdCiutada then
    RSP := 'ERROR:002';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_NomCiutada then
    RSP := 'ERROR:003';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_CognomsCiutada then
    RSP := 'ERROR:004';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_SexeCiutada then
    RSP := 'ERROR:005';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END CreateCitizen;
/

-- ---------------------------------------------------------------------------
-- Procediment per esborrar un ciutad de la base de dades.
-- Parmetres:
-- pCountryCode       IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber          IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP                OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE RemoveCitizen
(
  pCountryCode IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdNumber IN system.CIUTADANS."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_IdCiutada EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'RemoveCitizen';
  vParametres := pCountryCode || ', ' || pIdNumber;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    if (pIdNumber IS NULL or pIdNumber = '') then
      raise error_IdCiutada; 
    end if;
      
    -- Crear el ciutad.
    procEliminarCiutada(pCountryCode, pIdNumber, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_IdCiutada then
    RSP := 'ERROR:002';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END RemoveCitizen;
/

-- ---------------------------------------------------------------------------
-- Procediment per modificar les dades d'un ciutad da la base de dades.
-- Parmetres:
-- pCountryCode       IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber          IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pFirstName         IN VARCHAR2   Nom del ciutad.
-- pLastName          IN VARCHAR2   Cognoms del ciutad.
-- pSex               IN CHAR       Sexe del ciutad. Noms els valors H (home) i D (dona).
-- pEmail             IN VARCHAR2   Adrea de correu electrnic del ciutad. Pot ser null.
-- pDataNaixement     IN DATE       Data de naixement.
-- RSP                OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE UpdateCitizen
(
  pCountryCode IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdNumber IN system.CIUTADANS."IDCIUTADA"%TYPE,
  pFirstName IN system.CIUTADANS."NOM"%TYPE,
  pLastName IN system.CIUTADANS."COGNOMS"%TYPE,
  pSex IN system.CIUTADANS."SEXE"%TYPE,
  pEmail IN system.CIUTADANS."EMAIL"%TYPE,
  pBirthday IN system.CIUTADANS."DATANAIXEMENT"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_IdCiutada EXCEPTION;
  error_NomCiutada EXCEPTION;
  error_CognomsCiutada EXCEPTION;
  error_SexeCiutada EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'UpdateCitizen';
  vParametres := pCountryCode || ', ' || pIdNumber || ', ' || pFirstName || ', ' || pLastName || ', ' 
                 || pSex || ', ' || pEmail || ', ' || pBirthday;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    if (pIdNumber IS NULL or pIdNumber = '') then
      raise error_IdCiutada; 
    elsif (pFirstName IS NULL or pFirstName = '') then
      raise error_NomCiutada; 
    elsif (pLastName IS NULL or pLastName = '') then
      raise error_CognomsCiutada; 
    elsif (pSex IS NULL or (pSex <> 'H' and pSex <> 'D')) then
      raise error_SexeCiutada; 
    end if;
      
    -- Actualitzar les dades del ciutad.
    procActualitzarCiutada(pCountryCode, pIdNumber, pFirstName, pLastName, pSex, pEmail, pBirthday, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_IdCiutada then
    RSP := 'ERROR:002';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_NomCiutada then
    RSP := 'ERROR:003';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_CognomsCiutada then
    RSP := 'ERROR:004';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_SexeCiutada then
    RSP := 'ERROR:005';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END UpdateCitizen;
/

-- ---------------------------------------------------------------------------
-- Procediment per afegir dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pDate            IN DATE       Data a que fan referncia les dades.
-- pCountryCode     IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pSteps           IN INTEGER    Nmero de passes realitzades.
-- pCalories        IN INTEGER    Nmero de calores gastades.
-- pDowntime        IN INTEGER    Nmero d'hores de poca o nula activitat.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE AddPhysicalActivityData
(
  pDate IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCountryCode IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdNumber IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  pSteps IN system.ACTIVITATFISICA."PASSES"%TYPE,
  pCalories IN system.ACTIVITATFISICA."CALORIES"%TYPE,
  pDonwtime IN system.ACTIVITATFISICA."HORESINACTIVITAT"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
  error_DadesIncorrectes EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'AddPhysicalActivityData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber || ', ' || pSteps || ', ' || pCalories || ', ' || pDonwtime;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  elsif (pSteps < 0 or pCalories < 0) then
    raise error_DadesIncorrectes;
  elsif (pDonwtime < 0 or pDonwtime > 24) then
    raise error_DadesIncorrectes;
  end if;
  
  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;
  
  if (RSP = 'OK') then
    -- Actualitzar les dades.
    procAfegirActivitatFisica(pDate, pCountryCode, pIdNumber, pSteps, pCalories, pDonwtime, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_DadesIncorrectes then
    RSP := 'ERROR:016';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END AddPhysicalActivityData;
/

-- ---------------------------------------------------------------------------
-- Procediment per eliminar dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pDate                IN DATE       Data a que fan referncia les dades.
-- pCountryCode         IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber            IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP                  OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE RemovePhysicalActivityData
(
  pDate IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCountryCode IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdNumber IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'RemovePhysicalActivityData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  end if;

  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Actualitzar les dades del ciutad.
    procEliminarActivitatFisica(pDate, pCountryCode, pIdNumber, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END RemovePhysicalActivityData;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pDate              IN DATE       Data a que fan referncia les dades.
-- pCountryCode       IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber          IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pSteps             IN INTEGER    Nmero de passes realitzades.
-- pCalories          IN INTEGER    Nmero de calores gastades.
-- HoresInactivitat   IN INTEGER    Nmero d'hores de poca o nula activitat.
-- RSP                OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE UpdatePhysicalActivityData
(
  pDate IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCountryCode IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdNumber IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  pSteps IN system.ACTIVITATFISICA."PASSES"%TYPE,
  pCalories IN system.ACTIVITATFISICA."CALORIES"%TYPE,
  pDonwtime IN system.ACTIVITATFISICA."HORESINACTIVITAT"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
  error_DadesIncorrectes EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'UpdatePhysicalActivityData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber || ', ' || pSteps || ', ' || pCalories || ', ' || pDonwtime;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  elsif (pSteps < 0 or pCalories < 0) then
    raise error_DadesIncorrectes;
  elsif (pDonwtime < 0 or pDonwtime > 24) then
    raise error_DadesIncorrectes;
  end if;

  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Actualitzar les dades del ciutad.
    procActualitzaActivitatFisica(pDate, pCountryCode, pIdNumber, pSteps, pCalories, pDonwtime, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_DadesIncorrectes then
    RSP := 'ERROR:016';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END UpdatePhysicalActivityData;
/

-- ---------------------------------------------------------------------------
-- Procediment per afegir dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pDate            IN DATE       Data a que fan referncia les dades.
-- pCountryCode     IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pDiseaseCode     IN INTEGER    Codi de la malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE AddHealthData
(
  pDate IN system.ESTATSALUT."DATA"%TYPE,
  pCountryCode IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdNumber IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pDiseaseCode IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'AddHealthData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber || ', ' || pDiseaseCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  end if;

  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si la malaltia existeix.
    procVerificarCodiMalaltia(pDiseaseCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Actualitzar les dades del ciutad.
    procAfegirDadesSalut(pDate, pCountryCode, pIdNumber, pDiseaseCode, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END AddHealthData;
/

-- ---------------------------------------------------------------------------
-- Procediment per eliminar dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pDate            IN DATE       Data a que fan referncia les dades.
-- pCountryCode     IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pDiseaseCode     IN INTEGER    Codi de la malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE RemoveHealthData
(
  pDate IN system.ESTATSALUT."DATA"%TYPE,
  pCountryCode IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdNumber IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pDiseaseCode IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'RemoveHealthData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber || ', ' || pDiseaseCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si la malaltia existeix.
    procVerificarCodiMalaltia(pDiseaseCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Actualitzar les dades del ciutad.
    procEliminarDadesSalut(pDate, pCountryCode, pIdNumber, pDiseaseCode, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END RemoveHealthData;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pDate            IN DATE       Data a que fan referncia les dades.
-- pCountryCode     IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pOldDiseaseCode  IN INTEGER    Codi de la antiga malatia que ha sofert.
-- pNewDiseaseCode  IN INTEGER    Codi de la nova malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE UpdateHealthData
(
  pDate IN system.ESTATSALUT."DATA"%TYPE,
  pCountryCode IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdNumber IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pOldDiseaseCode IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  pNewDiseaseCode IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_DataIncorrecta EXCEPTION;
  error_CodiMalaltia EXCEPTION;
BEGIN
  -- Gravar un registre de log de entrada.
  vProcediment := 'UpdateHealthData';
  vParametres := pDate || ', ' || pCountryCode || ', ' || pIdNumber || ', ' || pOldDiseaseCode || ', ' || pNewDiseaseCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pDate IS NULL) then
    raise error_DataIncorrecta; 
  end if;
  if (pOldDiseaseCode = pNewDiseaseCode) then
    raise error_CodiMalaltia;
  end if;  
    
  -- Verificar si el codi de pas s vlid.
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Verificar si la malaltia existeix.
    procVerificarCodiMalaltia(pOldDiseaseCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Verificar si la malaltia existeix.
    procVerificarCodiMalaltia(pNewDiseaseCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Verificar si el ciutad existeix i no est donat de baixa.
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;
  
  if (RSP = 'OK') then
    -- Actualitzar les dades del ciutad.
    procActualitzarDadesSalut(pDate, pCountryCode, pIdNumber, pOldDiseaseCode, pNewDiseaseCode, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_DataIncorrecta then
    RSP := 'ERROR:018';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_CodiMalaltia then
    RSP := 'ERROR:019';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END UpdateHealthData;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de ciutadans d'un
-- pas que han aportat dades al sistema en un any determinat.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de ciutadans.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetNumberOfCitizens
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetNumberOfCitizens';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(1, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetNumberOfCitizens;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de calories gastades
-- pels ciutadans del pas.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de calories gastades.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetExpendedCalories
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetExpendedCalories';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(2, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetExpendedCalories;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de passes realitzades
-- pels ciutadans del pas.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de passes.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetStepsWalked
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetStepsWalked';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(3, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetStepsWalked;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero d'hores d'inactivitat
-- pels ciutadans del pas.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero d'hores d'inactivitat.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetDowntime
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetDowntime';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(4, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetDowntime;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de ciutadans del pas
-- als quals s'els hi ha diagnosticat una malaltia cardaca.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de ciutadans diagnosticats.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetHeartDiseases
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetHeartDiseases';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(5, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetHeartDiseases;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de ciutadans del pas
-- als quals s'els hi ha diagnosticat una malaltia cancergena.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de ciutadans diagnosticats.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetCancerDiseases
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetCancerDiseases';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(6, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetCancerDiseases;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de ciutadans del pas
-- als quals s'els hi ha diagnosticat una malaltia degenerativa del sistema nervis.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de ciutadans diagnosticats.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetDegenerativeDiseases
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR2
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetDegenerativeDiseases';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(7, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetDegenerativeDiseases;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del nmero de malalties lleus 
-- diagnosticat als ciutadans del pas.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCode     IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER          Nmero de malalties lleus diagnosticats.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION GetMinorIllness
(
  pYear IN NUMBER,
  pCountryCode IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR
) RETURN INTEGER
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
  valor INTEGER;
BEGIN
  valor := 0;
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetMinorIllness';
  vParametres := pYear || ', ' || pCountryCode;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCode <> 'UE') then
    procVerificarCodiPais(pCountryCode, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    valor := funcObtenirValorEstadistica(8, pYear, pCountryCode, RSP);
  end if;  
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  return valor;

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
    return 0;
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
    return 0;
        
END GetMinorIllness;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del ciutad que ms calories 
-- ha gastat.
-- Parmetres:
-- pYear            IN NUMBER     Any a consultar.
-- pCountryCodeE    IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- pCountryCodeC    IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- Valor de retorn:
-- valor            INTEGER       Nmero de malalties lleus diagnosticats.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE GetMostSpenderCitizen
(
  pYear IN NUMBER,
  pCountryCodeE IN system.ESTADISTICACALORIES."CODIPAISE"%TYPE,
  pCountryCodeC OUT system.ESTADISTICACALORIES."CODIPAISC"%TYPE,
  pIdCitizen OUT system.ESTADISTICACALORIES."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  error_AnyIncorrecte EXCEPTION;
BEGIN
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'GetMostSpenderCitizen';
  vParametres := pYear || ', ' || pCountryCodeE;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
    
  -- Verificar si el codi de pas s vlid.
  if (pCountryCodeE <> 'UE') then
    procVerificarCodiPais(pCountryCodeE, RSP);
  end if;  
  if (RSP = 'OK') then
    -- Obtenir la dada estadstica demanada.
    procObtenirCiutadaEstadistica(9, pYear, pCountryCodeE, pCountryCodeC, pIdCitizen, RSP);
  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END GetMostSpenderCitizen;
/

-- ---------------------------------------------------------------------------
-- Funci per obtenir la llista dels pasos de la Uni Europea.
-- Valor de retorn:
-- Taula            typeCountryTable    Taula de pasos.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeCountryRecord
  IS OBJECT
  (
    CountryCode CHAR(2),
    Name VARCHAR2(40)
  );
/

CREATE OR REPLACE TYPE typeCountryTable
  IS TABLE OF system."TYPECOUNTRYRECORD";
/

CREATE OR REPLACE FUNCTION GetCountriesList
RETURN system."TYPECOUNTRYTABLE"
IS
  CountryTable typeCountryTable;
  cursor CountryCursor is
    select * from system."VWLLISTATPAISOS";
  pais CountryCursor%ROWTYPE;
  indice NUMBER;
BEGIN
  -- Crear la collecci.
  CountryTable := typeCountryTable();
  
  -- Obtenir la llista dels pasos.
  open CountryCursor;
  loop
    fetch CountryCursor into pais;

    exit when CountryCursor%NOTFOUND;
    
    CountryTable.extend;
    indice := CountryTable.count;
    CountryTable(indice) := typeCountryRecord( pais."COUNTRYCODE", pais."NAME");
    
  end loop;
  close CountryCursor;
  
  -- I retornar la llista.
  return CountryTable;    

EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('GetCountriesList: Error ' || SQLCODE || ' - ' || SQLERRM);
    return CountryTable;
        
END GetCountriesList;
/

-- ---------------------------------------------------------------------------
-- Funci per obtenir la llista dels tipus de malalties suportades pel sistema.
-- Valor de retorn:
-- Taula            typeDiseaseTable    Taula de tipus de malalties.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeDiseaseRecord 
  IS OBJECT
  (
    DiseaseCode INTEGER,
    Description VARCHAR2(200)
  );
/

CREATE OR REPLACE TYPE typeDiseaseTable
  IS TABLE OF system."TYPEDISEASERECORD";
/

CREATE OR REPLACE FUNCTION GetDiseasesList
RETURN system."TYPEDISEASETABLE"
IS
  DiseaseTable typeDiseaseTable;
  cursor DiseaseCursor is
    select * from system."VWLLISTATMALALTIES";
  malaltia DiseaseCursor%ROWTYPE;
  indice NUMBER;
BEGIN
  -- Crear la collecci.
  DiseaseTable := typeDiseaseTable();
  
  -- Obtenir la llista dels tipus de malalties.
  open DiseaseCursor;
  loop
    fetch DiseaseCursor into malaltia;
    
    exit when DiseaseCursor%NOTFOUND;
    
    DiseaseTable.extend;
    indice := DiseaseTable.count;
    DiseaseTable(indice) := typeDiseaseRecord( malaltia."DISEASECODE", malaltia."DESCRIPTION");
    
  end loop;
  close DiseaseCursor;

  -- I retornar la llista.
  return DiseaseTable;    

EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('GetDiseasesList: Error ' || SQLCODE || ' - ' || SQLERRM);
    return DiseaseTable;
        
END GetDiseasesList;
/

-- ---------------------------------------------------------------------------
-- Procediment que mostra la llista dels ciutadans d'un pas d'una determinada
-- edat. Per a cada ciutad mostrar les seves dades d'estat de salut.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas.
-- pEdat            IN NUMBER     Edat dels ciutadans que es busquen.
-- pLlindarCalories IN INTEGER    Llindar de calories.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE ShowCitizensHealthStateList
(
  pYear IN NUMBER,
  pCountryCode IN system.PAISOS."CODIPAIS"%TYPE,
  pAge IN NUMBER,
  pCaloriesThresold IN system.ACTIVITATFISICA."CALORIES"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  nomPais system.PAISOS."NOM"%TYPE;
  error_AnyIncorrecte EXCEPTION;
  error_EdatIncorrecta EXCEPTION;
  error_LlindarIncorrecte EXCEPTION;
BEGIN
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'ShowCitizensHealthStateList';
  vParametres := pYear || ', ' || pCountryCode || ', ' || pAge || ', ' || pCaloriesThresold;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
  if (pAge IS NULL or pAge < 0 or pAge > 150) then
    raise error_EdatIncorrecta; 
  end if;
  if (pCaloriesThresold IS NULL or pCaloriesThresold < 0) then
    raise error_LlindarIncorrecte; 
  end if;
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    -- Mostrar la capalera del llistat.
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('CONSULTA DE L''ESTAT DE SALUT DELS CIUTADANS');
    DBMS_OUTPUT.PUT_LINE('============================================');
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Obtenir el nom del pas.
    select "NAME" into nomPais 
      from TABLE(GetCountriesList())
      where "COUNTRYCODE" = pCountryCode;
    
    -- Mostrar la informaci dels parmetres del llistat.
    DBMS_OUTPUT.PUT_LINE('Any                 : ' || pYear );
    DBMS_OUTPUT.PUT_LINE('Pas                : ' || pCountryCode || ' - ' || nomPais );
    DBMS_OUTPUT.PUT_LINE('Edat                : ' || pAge );
    DBMS_OUTPUT.PUT_LINE('Llindar de calories : ' || pCaloriesThresold );
    
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Mostrar les columnes del llistat.
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Id Ciutad', 20, ' ' ), 1, 20) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Nom', 30, ' ' ), 1, 30) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Cognoms', 40, ' ' ), 1, 40) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Visites metge', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Mitja calories', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Any cardaca', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Any cncer', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Any degenerativa', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Malalt. lleus', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT_LINE('');

    DBMS_OUTPUT.PUT( RPAD( '-', 20, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 30, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 40, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    
    DBMS_OUTPUT.PUT_LINE('');

    -- Mostrar el cos del llistat.
    for fila in 
    (
      select * from TABLE(funcLlistatSalutCiutadans(pYear, pCountryCode, pAge, pCaloriesThresold))
        order by "IDCIUTADA"
    )
    loop
      
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."IDCIUTADA", 20, ' ' ), 1, 20) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."NOM", 30, ' ' ), 1, 30) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."COGNOMS", 40, ' ' ), 1, 40) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( TO_CHAR(fila."VISITESMETGE"), 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( TO_CHAR(fila."MITJACALORIES"), 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( NVL( TO_CHAR(fila."ANYCARDIACA"), ' '), 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( NVL( TO_CHAR(fila."ANYCANCER"), ' '), 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( NVL( TO_CHAR(fila."ANYDEGENERATIVA"), ' '), 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( TO_CHAR(fila."MALALTIESLLEUS"), 13, ' ' ), 1, 13) );
      
      DBMS_OUTPUT.PUT_LINE('');
        
    end loop;

    DBMS_OUTPUT.PUT_LINE('');

  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_EdatIncorrecta then
    RSP := 'ERROR:014';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_LlindarIncorrecte then
    RSP := 'ERROR:015';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END ShowCitizensHealthStateList;
/

-- ---------------------------------------------------------------------------
-- Procediment que mostra els valors totals mensuals de l'activitat fsica 
-- realitzada per un ciutad concret en un any concret.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas del ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE ShowCitizenMonthlyActivityList
(
  pYear IN NUMBER,
  pCountryCode IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdNumber IN system.CIUTADANS."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  
  error_AnyIncorrecte EXCEPTION;
BEGIN
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'ShowCitizenMonthlyActivityList';
  vParametres := pYear || ', ' || pCountryCode || ', ' || pIdNumber;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
  procVerificarCodiPais(pCountryCode, RSP);
  if (RSP = 'OK') then
    procVerificarCiutadaAlta(pCountryCode, pIdNumber, RSP);
  end if;
  if (RSP = 'OK') then
    -- Mostrar la capalera del llistat.
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('CONSULTA DE L''ACTIVITAT FSICA D''UN CIUTAD');
    DBMS_OUTPUT.PUT_LINE('=============================================');
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Mostrar la informaci dels parmetres del llistat.
    DBMS_OUTPUT.PUT_LINE('Any    : ' || pYear );
    DBMS_OUTPUT.PUT_LINE('Ciutad: ' || pCountryCode || ' - ' || pIdNumber );
    
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Mostrar les columnes del llistat.
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Mes', 20, ' ' ), 1, 20) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Passes', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Mitja calories', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Hores inactivitat', 13, ' ' ), 1, 13) );

    DBMS_OUTPUT.PUT_LINE('');

    DBMS_OUTPUT.PUT( RPAD( '-', 20, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
   
    DBMS_OUTPUT.PUT_LINE('');

    -- Mostrar el cos del llistat.
    for fila in 
    (
      select * from TABLE(funcActFisicaMensualCiutada(pYear, pCountryCode, pIdNumber))
        order by "NUMMES"
    )
    loop
      
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."NOMMES", 20, ' ' ), 1, 20) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."PASSES", 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."MITJACALORIES", 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."HORESINACTIVITAT", 13, ' ' ), 1, 13) );
      
      DBMS_OUTPUT.PUT_LINE('');
        
    end loop;

    DBMS_OUTPUT.PUT_LINE('');

  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END ShowCitizenMonthlyActivityList;
/

-- ---------------------------------------------------------------------------
-- Procediment que mostra la llista dels valors totals anuals de l'activitat fsica 
-- realitzada per tots els ciutadans de cada pas que tinguin una determinada edat
-- en un any concret.
-- pYear            IN NUMBER     Any de les dades que es volen consultar.
-- pAge             IN NUMBER     Edat mxima dels ciutadans que es volen consultar.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE ShowCountryActivityList
(
  pYear IN NUMBER,
  pAge IN NUMBER,
  RSP OUT VARCHAR2
)
IS
  vProcediment VARCHAR2(50);
  vParametres VARCHAR2(256);
  
  error_AnyIncorrecte EXCEPTION;
  error_EdatIncorrecta EXCEPTION;
BEGIN
  RSP := 'OK';
  
  -- Gravar un registre de log de entrada.
  vProcediment := 'ShowCountryActivityList';
  vParametres := pYear || ', ' || pAge;
  PROCGRAVARLOG(vProcediment, 'E', vParametres, '*');

  -- Validaci dels parmetres.
  if (pYear IS NULL or pYear <= 0) then
    raise error_AnyIncorrecte; 
  end if;
  if (pAge IS NULL or pAge < 0 or pAge > 150) then
    raise error_EdatIncorrecta; 
  end if;
  if (RSP = 'OK') then
    -- Mostrar la capalera del llistat.
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('CONSULTA DE L''ACTIVITAT FSICA A TOTS ELS PASOS');
    DBMS_OUTPUT.PUT_LINE('=================================================');
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Mostrar la informaci dels parmetres del llistat.
    DBMS_OUTPUT.PUT_LINE('Any  : ' || pYear );
    DBMS_OUTPUT.PUT_LINE('Edat : ' || pAge );
    
    DBMS_OUTPUT.PUT_LINE('');
    
    -- Mostrar les columnes del llistat.
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Codi', 4, ' ' ), 1, 4) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( RPAD( 'Pas', 40, ' ' ), 1, 40) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Mitja passes', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Mitja calories', 13, ' ' ), 1, 13) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( SUBSTR( LPAD( 'Mitja hores inactivitat', 13, ' ' ), 1, 13) );

    DBMS_OUTPUT.PUT_LINE('');

    DBMS_OUTPUT.PUT( RPAD( '-', 4, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 40, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
    DBMS_OUTPUT.PUT( RPAD( '-', 13, '-' ) );
    DBMS_OUTPUT.PUT(' ');
   
    DBMS_OUTPUT.PUT_LINE('');

    -- Mostrar el cos del llistat.
    for fila in 
    (
      select * from TABLE(funcActFisicaMitjaPaisos(pYear, pAge))
    )
    loop
      
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."CODIPAIS", 4, ' ' ), 1, 4) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( RPAD( fila."NOMPAIS", 40, ' ' ), 1, 40) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."MITJAPASSES", 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."MITJACALORIES", 13, ' ' ), 1, 13) );
      DBMS_OUTPUT.PUT(' ');
      DBMS_OUTPUT.PUT( SUBSTR( LPAD( fila."MITJAHORESINACTIVITAT", 13, ' ' ), 1, 13) );
      
      DBMS_OUTPUT.PUT_LINE('');
        
    end loop;

    DBMS_OUTPUT.PUT_LINE('');

  end if;
  
  -- Gravar un registre de log de sortida.
  PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);

EXCEPTION
  when error_AnyIncorrecte then
    RSP := 'ERROR:013';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when error_EdatIncorrecta then
    RSP := 'ERROR:014';
    PROCGRAVARLOG(vProcediment, 'S', vParametres, RSP);
  
  when others then
    RSP := 'ERROR:017';
    PROCGRAVARLOG(vProcediment, 'S', SQLCODE || ' - ' || SQLERRM, RSP);
        
END ShowCountryActivityList;
/

-- ***************************************************************************
-- Final del script.
-- ***************************************************************************
